@@ -1,3 +1,5 @@
+use assert_matches::assert_matches;
+
 use prae;
 
 prae::define! {
@@ -8,7 +10,7 @@ prae::define! {
 
 #[test]
 fn construction_fails_for_invalid_data() {
-    assert_eq!(Username::new("  ").unwrap_err(), prae::ValidationError);
+    assert_matches!(Username::new("  "), Err(prae::ConstructionError { .. }))
 }
 
 #[test]
@@ -20,10 +22,10 @@ fn construction_succeeds_for_valid_data() {
 #[test]
 fn mutation_fails_for_invalid_data() {
     let mut un = Username::new("user").unwrap();
-    assert_eq!(
-        un.try_mutate(|u| *u = "  ".to_owned()).unwrap_err(),
-        prae::ValidationError
-    );
+    assert_matches!(
+        un.try_mutate(|u| *u = "  ".to_owned()),
+        Err(prae::MutationError { .. })
+    )
 }
 
 #[test]
@@ -38,4 +40,4 @@ fn mutation_succeeds_for_valid_data() {
     let mut un = Username::new("user").unwrap();
     assert!(un.try_mutate(|u| *u = "  new user  ".to_owned()).is_ok());
     assert_eq!(un.get(), "new user");
-}
\ No newline at end of file
+}
