@@ -1,3 +1,5 @@
+use assert_matches::assert_matches;
+
 use prae;
 
 prae::define! {
@@ -5,9 +7,28 @@ prae::define! {
     ensure |u| !u.is_empty()
 }
 
+#[test]
+fn construction_error_formats_correctly() {
+    let err = Username::new("").unwrap_err();
+    assert_eq!(
+        err.to_string(),
+        "failed to create Username from value \"\": provided value is invalid"
+    );
+}
+
+#[test]
+fn mutation_error_formats_correctly() {
+    let mut un = Username::new("user").unwrap();
+    let err = un.try_mutate(|u| *u = "".to_owned()).unwrap_err();
+    assert_eq!(
+        err.to_string(),
+        "failed to mutate Username from value \"user\" to \"\": provided value is invalid"
+    );
+}
+
 #[test]
 fn construction_fails_for_invalid_data() {
-    assert_eq!(Username::new("").unwrap_err(), prae::ValidationError);
+    assert_matches!(Username::new(""), Err(prae::ConstructionError { .. }));
 }
 
 #[test]
@@ -19,10 +40,10 @@ fn construction_succeeds_for_valid_data() {
 #[test]
 fn mutation_fails_for_invalid_data() {
     let mut un = Username::new("user").unwrap();
-    assert_eq!(
-        un.try_mutate(|u| *u = "".to_owned()).unwrap_err(),
-        prae::ValidationError
-    );
+    assert_matches!(
+        un.try_mutate(|u| *u = "".to_owned()),
+        Err(prae::MutationError { .. })
+    )
 }
 
 #[test]
@@ -37,4 +58,4 @@ fn mutation_succeeds_for_valid_data() {
     let mut un = Username::new("user").unwrap();
     assert!(un.try_mutate(|u| *u = " new user ".to_owned()).is_ok());
     assert_eq!(un.get(), " new user ");
-}
\ No newline at end of file
+}
